package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.JavadocArgumentExpression;
import org.eclipse.jdt.internal.compiler.ast.MessageSend;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class JavadocMessageSend extends MessageSend {
  public int tagSourceStart;
  
  public int tagSourceEnd;
  
  public int tagValue;
  
  public JavadocMessageSend(char[] name, long pos) {
    this.selector = name;
    this.nameSourcePosition = pos;
    this.sourceStart = (int)(this.nameSourcePosition >>> 32L);
    this.sourceEnd = (int)this.nameSourcePosition;
    this.bits |= 0x8000;
  }
  
  public JavadocMessageSend(char[] name, long pos, JavadocArgumentExpression[] arguments) {
    this(name, pos);
    this.arguments = (Expression[])arguments;
  }
  
  private TypeBinding internalResolveType(Scope scope) {
    this.constant = Constant.NotAConstant;
    if (this.receiver == null) {
      this.actualReceiverType = (TypeBinding)scope.enclosingReceiverType();
    } else if (scope.kind == 3) {
      this.actualReceiverType = this.receiver.resolveType((ClassScope)scope);
    } else {
      this.actualReceiverType = this.receiver.resolveType((BlockScope)scope);
    } 
    boolean hasArgsTypeVar = false;
    if (this.arguments != null) {
      this.argumentsHaveErrors = false;
      int length = this.arguments.length;
      this.argumentTypes = new TypeBinding[length];
      for (int i = 0; i < length; i++) {
        Expression argument = this.arguments[i];
        if (scope.kind == 3) {
          this.argumentTypes[i] = argument.resolveType((ClassScope)scope);
        } else {
          this.argumentTypes[i] = argument.resolveType((BlockScope)scope);
        } 
        if (this.argumentTypes[i] == null) {
          this.argumentsHaveErrors = true;
        } else if (!hasArgsTypeVar) {
          hasArgsTypeVar = this.argumentTypes[i].isTypeVariable();
        } 
      } 
      if (this.argumentsHaveErrors)
        return null; 
    } 
    if (this.actualReceiverType == null)
      return null; 
    this.actualReceiverType = scope.environment().convertToRawType(this.receiver.resolvedType, true);
    ReferenceBinding enclosingType = scope.enclosingReceiverType();
    if (enclosingType != null && enclosingType.isCompatibleWith(this.actualReceiverType))
      this.bits |= 0x4000; 
    if (this.actualReceiverType.isBaseType()) {
      scope.problemReporter().javadocErrorNoMethodFor(this, this.actualReceiverType, this.argumentTypes, scope.getDeclarationModifiers());
      return null;
    } 
    this.binding = scope.getMethod(this.actualReceiverType, this.selector, this.argumentTypes, (InvocationSite)this);
    if (!this.binding.isValidBinding()) {
      TypeBinding enclosingTypeBinding = this.actualReceiverType;
      MethodBinding methodBinding = this.binding;
      while (!methodBinding.isValidBinding() && (enclosingTypeBinding.isMemberType() || enclosingTypeBinding.isLocalType())) {
        ReferenceBinding referenceBinding = enclosingTypeBinding.enclosingType();
        methodBinding = scope.getMethod((TypeBinding)referenceBinding, this.selector, this.argumentTypes, (InvocationSite)this);
      } 
      if (methodBinding.isValidBinding()) {
        this.binding = methodBinding;
      } else {
        enclosingTypeBinding = this.actualReceiverType;
        MethodBinding contructorBinding = this.binding;
        if (!contructorBinding.isValidBinding() && CharOperation.equals(this.selector, enclosingTypeBinding.shortReadableName()))
          contructorBinding = scope.getConstructor((ReferenceBinding)enclosingTypeBinding, this.argumentTypes, (InvocationSite)this); 
        while (!contructorBinding.isValidBinding() && (enclosingTypeBinding.isMemberType() || enclosingTypeBinding.isLocalType())) {
          ReferenceBinding referenceBinding = enclosingTypeBinding.enclosingType();
          if (CharOperation.equals(this.selector, referenceBinding.shortReadableName()))
            contructorBinding = scope.getConstructor(referenceBinding, this.argumentTypes, (InvocationSite)this); 
        } 
        if (contructorBinding.isValidBinding())
          this.binding = contructorBinding; 
      } 
    } 
    if (!this.binding.isValidBinding()) {
      MethodBinding closestMatch;
      switch (this.binding.problemId()) {
        case 3:
        case 5:
        case 6:
        case 7:
          closestMatch = ((ProblemMethodBinding)this.binding).closestMatch;
          if (closestMatch != null)
            this.binding = closestMatch; 
          break;
      } 
    } 
    if (!this.binding.isValidBinding()) {
      if (this.receiver.resolvedType instanceof org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding)
        return null; 
      if (this.binding.declaringClass == null)
        if (this.actualReceiverType instanceof ReferenceBinding) {
          this.binding.declaringClass = (ReferenceBinding)this.actualReceiverType;
        } else {
          scope.problemReporter().javadocErrorNoMethodFor(this, this.actualReceiverType, this.argumentTypes, scope.getDeclarationModifiers());
          return null;
        }  
      scope.problemReporter().javadocInvalidMethod(this, this.binding, scope.getDeclarationModifiers());
      if (this.binding instanceof ProblemMethodBinding) {
        MethodBinding closestMatch = ((ProblemMethodBinding)this.binding).closestMatch;
        if (closestMatch != null)
          this.binding = closestMatch; 
      } 
      return this.resolvedType = (this.binding == null) ? null : this.binding.returnType;
    } 
    if (hasArgsTypeVar) {
      ProblemMethodBinding problemMethodBinding = new ProblemMethodBinding(this.binding, this.selector, this.argumentTypes, 1);
      scope.problemReporter().javadocInvalidMethod(this, (MethodBinding)problemMethodBinding, scope.getDeclarationModifiers());
    } else if (this.binding.isVarargs()) {
      int length = this.argumentTypes.length;
      if (this.binding.parameters.length != length || !this.argumentTypes[length - 1].isArrayType()) {
        ProblemMethodBinding problemMethodBinding = new ProblemMethodBinding(this.binding, this.selector, this.argumentTypes, 1);
        scope.problemReporter().javadocInvalidMethod(this, (MethodBinding)problemMethodBinding, scope.getDeclarationModifiers());
      } 
    } else {
      int length = this.argumentTypes.length;
      for (int i = 0; i < length; i++) {
        if (TypeBinding.notEquals(this.binding.parameters[i].erasure(), this.argumentTypes[i].erasure())) {
          ProblemMethodBinding problemMethodBinding = new ProblemMethodBinding(this.binding, this.selector, this.argumentTypes, 1);
          scope.problemReporter().javadocInvalidMethod(this, (MethodBinding)problemMethodBinding, scope.getDeclarationModifiers());
          break;
        } 
      } 
    } 
    if (isMethodUseDeprecated(this.binding, scope, true))
      scope.problemReporter().javadocDeprecatedMethod(this.binding, (ASTNode)this, scope.getDeclarationModifiers()); 
    return this.resolvedType = this.binding.returnType;
  }
  
  public boolean isSuperAccess() {
    return ((this.bits & 0x4000) != 0);
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    if (this.receiver != null)
      this.receiver.printExpression(0, output); 
    output.append('#').append(this.selector).append('(');
    if (this.arguments != null)
      for (int i = 0; i < this.arguments.length; i++) {
        if (i > 0)
          output.append(", "); 
        this.arguments[i].printExpression(0, output);
      }  
    return output.append(')');
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    return internalResolveType((Scope)scope);
  }
  
  public TypeBinding resolveType(ClassScope scope) {
    return internalResolveType((Scope)scope);
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    if (visitor.visit(this, blockScope)) {
      if (this.receiver != null)
        this.receiver.traverse(visitor, blockScope); 
      if (this.arguments != null) {
        int argumentsLength = this.arguments.length;
        for (int i = 0; i < argumentsLength; i++)
          this.arguments[i].traverse(visitor, blockScope); 
      } 
    } 
    visitor.endVisit(this, blockScope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope scope) {
    if (visitor.visit(this, scope)) {
      if (this.receiver != null)
        this.receiver.traverse(visitor, scope); 
      if (this.arguments != null) {
        int argumentsLength = this.arguments.length;
        for (int i = 0; i < argumentsLength; i++)
          this.arguments[i].traverse(visitor, scope); 
      } 
    } 
    visitor.endVisit(this, scope);
  }
}
